import requests
import time


def get(url, headers=None):
    """
    Custom get request.

    Args:
        url: Request needs a url.

    Returns:
        Return the response.
    """
    res = requests.get(url, headers=headers)
    request_count = 0

    while res.status_code != 200 and request_count <= 10:
        time.sleep(1.5)
        res = requests.get(url, headers=headers)
        request_count += 1

    return res



import time
import requests
from bs4 import BeautifulSoup

from .shortcuts import get


class Anime:
    def __init__(self, url):
        headers = {
            'User-Agent': 'Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:68.0) Gecko/20100101 Firefox/68.0'
        }
        res = get(url, headers=headers)
        self._soup = BeautifulSoup(res.text, features='lxml')
        print(f'Scraped {url}')

    @property
    def title(self):
        title = ''

        try:
            span = self._soup.find('span', {'itemprop': 'name'})
            title = span.text
        except:
            print('Anime: Skipping title.')

        return title

    @property
    def english_title(self):
        title = ''

        try:
            divs = self._soup.find_all('div', {'class': 'spaceit_pad'})

            for div in divs:
                if 'English:' in div.text:
                    title = div.text.replace('English:', '').rstrip().lstrip()
                    break
        except:
            print('Anime: Skipping english title.')

        return title

    @property
    def japanese_title(self):
        title = ''

        try:
            divs = self._soup.find_all('div', {'class': 'spaceit_pad'})

            for div in divs:
                if 'Japanese:' in div.text:
                    title = div.text.replace('Japanese:', '').rstrip().lstrip()
                    break
        except:
            print('Anime: Skipping japanese title .')

        return title

    @property
    def synonyms(self):
        syn = ''

        try:
            divs = self._soup.find_all('div', {'class': 'spaceit_pad'})

            for div in divs:
                if 'Synonyms:' in div.text:
                    syn = div.text.replace(
                        'Synonyms:', '').rstrip().lstrip()
                    break
        except:
            print('Anime: Skipping synonyms.')

        return syn

    @property
    def synopsis(self):
        synopsis = ''

        try:
            span = self._soup.find('span', {'itemprop': 'description'})
            synopsis = span.text
        except:
            print('Anime: Skipping synopsis.')

        return synopsis

    @property
    def animetype(self):
        atype = ''

        try:
            divs = self._soup.find(
                'div', {'class': 'js-scrollfix-bottom'}).find_all('div')

            for div in divs:
                if 'Type:' in div.text:
                    atype = div.text.replace('Type:', '').rstrip().lstrip()
                    break
        except:
            print('Anime: Skipping anime type.')

        return atype

    @property
    def episodes(self):
        eps = ''

        try:
            divs = self._soup.find(
                'div', {'class': 'js-scrollfix-bottom'}).find_all('div', {'class': 'spaceit'})

            for div in divs:
                if 'Episodes:' in div.text:
                    eps = div.text.replace('Episodes:', '').rstrip().lstrip()
                    break
        except:
            print('Anime: Skipping episodes.')

        return eps

    @property
    def genres(self):
        genres = ''

        try:
            divs = self._soup.find(
                'div', {'class': 'js-scrollfix-bottom'}).find_all('div')

            for div in divs:
                if 'Genres:' in div.text:
                    genres = div.text.replace('Genres:', '').rstrip().lstrip()
                    break
        except:
            print('Anime: Skipping genres.')

        return genres

    @property
    def poster(self):
        poster = ''

        try:
            img = self._soup.find('img', {'class': 'ac'})
            poster = img['src']
        except:
            print('Anime: Skipping poster.')

        return poster

    @property
    def trailer(self):
        trailer = ''

        try:
            a = self._soup.find(
                'a', {'class': 'iframe js-fancybox-video video-unit promotion'})
            trailer = a['href']
        except:
            print('Anime: Skipping trailer.')

        return trailer

    def get_data(self):
        """
        Gets the full anime data in json.

        Returns:
            Dict object.
        """
        data = {
            'title': self.title,
            'english_title': self.english_title,
            'japanese_title': self.japanese_title,
            'synonyms': self.synonyms,
            'synopsis': self.synopsis,
            'type': self.animetype,
            'episodes': self.episodes,
            'genres': self.genres,
            'poster': self.poster,
            'trailer': self.trailer
        }

        return data

    def __repr__(self):
        return f'<models.Anime {id(self)}>'


class Character:
    def __init__(self, url):
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:68.0) Gecko/20100101 Firefox/68.0'
        }
        res = get(url, headers=self.headers)
        self._soup = BeautifulSoup(res.text, features='lxml')
        self._url = url
        print(f'Scraped {url}')

    @property
    def name(self):
        name = ''

        try:
            div = self._soup.find('div', {'id': 'content'}).find(
                'td', {'style': 'padding-left: 5px;', 'valign': 'top'}).find('div', {'class': 'normal_header'})
            name = div.text
        except:
            print('Character: Skipping name.')

        return name

    @property
    def poster(self):
        poster = ''

        try:
            img = self._soup.find('div', {'id': 'content'}).find('img')
            poster = img['src']
        except:
            print('Character: Skipping poster.')

        return poster

    def get_gallery(self):
        url = self._url + '/pictures'
        res = get(url, headers=self.headers)
        soup = BeautifulSoup(res.text, features='lxml')

        try:
            imgs = soup.find('div', {'id': 'content'}).find(
                'td', {'style': 'padding-left: 5px;', 'valign': 'top'}).find('table').find_all('img')
            gallery = []

            for i, img in enumerate(imgs):
                try:
                    gallery.append(img['src'])
                except:
                    print(f'Character: Skipping gallery image #{i + 1}.')
        except:
            print('Character: Skipping Gallery.')

        return gallery

    def __repr__(self):
        return f'<models.Character {id(self)}>'



from setuptools import setup, find_packages


with open("README.md", "r") as fh:
    long_description = fh.read()

    setup(name='pymalscraper',
          version='2.0.1',
          author='prinsepipo',
          author_email='prinse.sanchez@gmail.com',
          description='Simple Anime Web Scraper.',
          long_description=long_description,
          long_description_content_type="text/markdown",
          url='https://github.com/prinsepipo/anime-scraper',
          license='MIT',
          packages=find_packages(),
          install_requires=[
              'requests',
              'BeautifulSoup4',
              'lxml'
          ],
          python_requires='>=3',
          zip_safe=False)



from .models import Anime, Character
from .shortcuts import get

import requests
from bs4 import BeautifulSoup

import time
from concurrent.futures import ThreadPoolExecutor


class Scraper:
    """
    Scrape data from https://myanimelist.net/.
    """

    def __init__(self):
        """
        Initialize the scraper.
        """
        # MAL search url
        self.MAL_ANIME_URL = 'https://myanimelist.net/anime.php?q='
        self.MAL_CHAR_URL = 'https://myanimelist.net/character.php?q='
        self.headers = {
            'User-Agent': 'Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:68.0) Gecko/20100101 Firefox/68.0'
        }

    def search_anime(self, name):
        """
        Args:
            name: Name of anime.

        Returns:
            Return a list of tuple that contains the nane and url of the anime.
        """
        url = self.MAL_ANIME_URL + str(name)
        res = get(url, self.headers)

        if res.status_code != 200:
            raise Exception(f'Response code {res.status_code}.')

        soup = BeautifulSoup(res.text, features='lxml')
        queryset = []

        try:
            div = soup.find(
                'div', {'class': 'js-categories-seasonal js-block-list list'})
            table_rows = div.find_all('tr')[1:6]

            for row in table_rows:
                td = row.find_all('td')[1]
                a = td.find('a')
                queryset.append((a.text, a['href']))
        except Exception as e:
            print(f'Parse Error.\n{e}')

        return queryset

    def get_anime(self, name=None, url=None):
        """
        Get the anime model data. Method only accepts one parameter, either name or url.

        Args:
            name: The name of the anime.
            url: The url of the anime.

        Returns:
            Return the Anime model data.
        """
        anime_url = None

        if name and not url:
            anime_url = self.search_anime(str(name))[0][1]
        elif url and not name:
            anime_url = url
        else:
            raise ValueError('Method needs one parameter, anime or url.')

        return Anime(anime_url)

    def get_all_anime(self, start=0, end=10000):
        """
        Scrape all the anime from the website. Scrapes 50 anime from start to end.
        Note: Stoping the process will result to loss of data.

        Args:
            start: Where to begin scraping.
            end: Where to end scraping.

        Returns:
            Return a list of Anime model data.
        """
        total_anime = end - 50
        count = start
        links = []

        while count <= total_anime:
            url = f'https://myanimelist.net/topanime.php?limit={count}'
            res = get(url, headers=self.headers)
            soup = BeautifulSoup(res.text, features='lxml')

            try:
                aa = soup.find_all(
                    'a', {'class': 'hoverinfo_trigger fl-l fs14 fw-b'})

                for a in aa:
                    link = a['href']

                    if link:
                        links.append(link)
            except Exception as e:
                print(e)

            count += 50

        print(f'Scraping animes total of {len(links)}')

        with ThreadPoolExecutor() as executor:
            animes = executor.map(Anime, links)

        return list(animes)

    def search_character(self, name):
        """
        Args:
            name: Name of the character.

        Returns:
            Return a list of tuple that contains the name and url of the character.
        """
        url = self.MAL_CHAR_URL + str(name)
        res = get(url, headers=self.headers)

        if res.status_code != 200:
            raise Exception(f'Response code {res.status_code}.')

        soup = BeautifulSoup(res.text, features='lxml')
        queryset = []

        try:
            div = soup.find('div', {'id': 'content'})
            table = div.find('table')
            table_rows = table.find_all('tr')[1:6]

            for row in table_rows:
                td = row.find_all('td')[1]
                a = td.find('a')
                queryset.append((a.text, a['href']))
        except Exception as e:
            print(f'Parse Error.\n{e}')

        return queryset

    def get_character(self, name=None, url=None):
        """
        Get the character model data. Method only accepts one parameter, either name or url.

        Args:
            name: Name of the character.
            url: Url of the character.

        Returns:
            Return the Character model data.
        """
        char_url = None

        if name and not url:
            char_url = self.search_character(str(name))[0][1]
        elif url and not name:
            char_url = url
        else:
            raise ValueError('Method needs one parameter, anime or url.')

        return Character(char_url)

    def get_all_characters(self, start=0, end=10000):
        """
        Scrape all the character from the website. Scrapes 50 character from start to end.
        Note: Stoping the process will result to loss of data.

        Args:
            start: Where to begin scraping.
            end: Where to end scraping.

        Returns:
            Return a list of Character model data.
        """
        total_anime = end - 50
        count = start
        links = []

        while count <= total_anime:
            url = f'https://myanimelist.net/character.php?limit={count}'
            res = get(url, headers=self.headers)
            soup = BeautifulSoup(res.text, features='lxml')

            try:
                aa = soup.find('div', {'id': 'content'}).find_all(
                    'a', {'class': 'fs14 fw-b'})

                for a in aa:
                    link = a['href']

                    if link:
                        links.append(link)
            except Exception as e:
                print(e)

            count += 50

        print(f'Scraping animes total of {len(links)}')

        with ThreadPoolExecutor() as executor:
            chars = executor.map(Character, links)

        return list(chars)
